// Global implemented and various bug fixes by Trevor Hobson
var objGlobal = {
	nsIFilePicker: Components.interfaces.nsIFilePicker,
	objectionPrefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection."),
	objectStrings: '',
	optionflashAppDir: '',
	lsos: [],
	deletedLsos: [],
	count: 0,
	lsosTree: null,
	lastLsoSortColumn: '',  
	lastLsoSortAscending: false,

	lsosTreeView:
	{
		rowCount: 0,
		setTree: function(tree) {},
		getImageSrc: function(row, column) {},
		getProgressMode: function(row, column) {},
		getCellValue: function(row, column) {},
		getCellText: function(row, column) {
			var rv="";
			if (column.id=="parentCol") // Added .id TH
			{
				rv = objGlobal.lsos[row].parent;
			}
			else if (column.id=="nameCol") // Added .id TH
			{
				rv = objGlobal.lsos[row].name;
			}
			return rv;
		},
		isSeparator: function(index) {return false;},
		isSorted: function() {return false;},
		isContainer: function(index) {return false;},
		cycleHeader: function(aColId, aElt) {},
		getRowProperties: function(row, props) {},
		getColumnProperties: function(column, props) {},
		getCellProperties: function(row, column, props) {},
	},

	objection_init: function()
	{
		// Read string bundle set
		objGlobal.objectStrings = document.getElementById('lsoStrings');

		// Reference the tree
		objGlobal.lsosTree = document.getElementById('lsosTree');

		// Display error if selected directory does not exist
		var checkFlashDir = GetFlashDir();
		if (checkFlashDir == "")
		{
			document.getElementById("appdir").value = objGlobal.objectStrings.getString("dirNotFound");;
		}
		else
		{
			document.getElementById("appdir").value = checkFlashDir.path;
			objGlobal.loadLsos(checkFlashDir);
		}
	},

	changeAppDir: function()
	{
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(objGlobal.nsIFilePicker);
		fp.init(window, objGlobal.objectStrings.getString("selectDirTitle"), objGlobal.nsIFilePicker.modeGetFolder);
		var rv = fp.show();
		if (rv == objGlobal.nsIFilePicker.returnOK)
		{
			var file = fp.file;
			if (document.getElementById("appdir").value != file.path)
			{
				document.getElementById("appdir").value = file.path;
				objGlobal.objectionPrefs.setCharPref("flashAppDir", file.path);
				objGlobal.loadLsos(file);
			}
		}
	},

	searchAutomatic: function()
	{
		// Display error if selected directory does not exist
		var checkFlashDir = GetFlashDir(true);
		if (checkFlashDir == "")
		{
			document.getElementById("appdir").value = objGlobal.objectStrings.getString("dirNotFound");;
		}
		else
		{
			document.getElementById("appdir").value = checkFlashDir.path;
			objGlobal.loadLsos(checkFlashDir);
		}
	},

	Lso: function(number, file, name, size, path, parent, pathFlash)
	{
		this.number = number;
		this.file = file;
		this.name = name;
		this.size = size;
		this.path = path;
		this.pathShort = '...' + path.substr(pathFlash.length);
		this.parent = parent;
		return this;
	},

	loadLsos: function(flashDir)
	{
		objGlobal.count = 0;
		objGlobal.lsos = new Array();
		objGlobal.loadLsosSub(flashDir, flashDir.path);
		objGlobal.lsosTreeView.rowCount = objGlobal.lsos.length;
// Work around for the problem below, sort the array first
		objGlobal.lastLsoSortAscending = objGlobal.SortTreeArray(objGlobal.lsos, 'parent', objGlobal.lastLsoSortColumn, objGlobal.lastLsoSortAscending);
		objGlobal.lastLsoSortColumn = 'parent';
// End workaround
		objGlobal.lsosTree.view = objGlobal.lsosTreeView;
/* Cannot figure out why FF3 returns null to the next line. The workaround is faster, but leaving this for future investigation
alert(objGlobal.lsosTree.view);  // [xpconnect wrapped nsITreeView] - null  (This line for testing only)
		objGlobal.LsoColumnSort('parent');
*/ 
		document.getElementById("removeAllLsos").disabled = objGlobal.lsos.length == 0;
	},

	loadLsosSub: function(currentDir, flashPath)
	{
		var entries = currentDir.directoryEntries;
		while(entries.hasMoreElements())
		{
			var entry = entries.getNext();
			entry.QueryInterface(Components.interfaces.nsIFile);
			if (entry.isFile())
			{
				var leaf  = entry.leafName;
				var dotIndex  = leaf.lastIndexOf('.');
				var extension = (dotIndex >= 0) ? leaf.substring(dotIndex+1) : "";
				if ((extension == "sol")||(extension == "fso"))
				{
					objGlobal.lsos[objGlobal.count] = new objGlobal.Lso(objGlobal.count++, entry, entry.leafName, entry.fileSize, entry.path, entry.parent.leafName, flashPath);
				}
			}
			else if (entry.isDirectory())
			{
				objGlobal.loadLsosSub(entry, flashPath);
			}
		}
	},

	LsoColumnSort: function(column)
	{
		objGlobal.lastLsoSortAscending = objGlobal.SortTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, column, objGlobal.lastLsoSortColumn, objGlobal.lastLsoSortAscending, true);
		objGlobal.lastLsoSortColumn = column;
	},

	LsoSelected: function()
	{
		var selections = objGlobal.GetTreeSelections(objGlobal.lsosTree);
		if (selections.length)
		{
			document.getElementById("removeLso").removeAttribute("disabled");
		}
		else
		{
		return true;
		}
		var indexvar = selections[0];
		if (indexvar >= objGlobal.lsos.length)
		{
			// Something got out of synch.  See bug 119812 for details (Included from original code, bug has been fixed but keep to be safe - TH)
			dump("Out of synch;see Mozilla bug 119812 to help.");
			return;
		}
		if (selections.length > 1)
		{
			objGlobal.SetLsoProperties("", "", "", "");
		}
		else
		{
			objGlobal.SetLsoProperties(objGlobal.lsos[indexvar].name, objGlobal.lsos[indexvar].parent, objGlobal.lsos[indexvar].size + " bytes", objGlobal.lsos[indexvar].pathShort);
		}
		return true;
	},

	HandleLsoKeyPress: function(e)
	{
		if (e.keyCode == 46)
		{
			objGlobal.DeleteLso();
		}
	},

	DeleteLso: function()
	{
		var toDelete = objGlobal.GetTreeSelections(objGlobal.lsosTree);
		for (var x=0;x<toDelete.length;x++)
		{
			objGlobal.lsos[toDelete[x]].file.remove(false);
		}
		objGlobal.DeleteSelectedItemFromTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, objGlobal.deletedLsos, "removeLso", "removeAllLsos");
		if (!objGlobal.lsos.length)
		{
			objGlobal.SetLsoProperties("", "", "", "");
		}
	},

	DeleteAllLsos: function()
	{
		for (var x=0;x<objGlobal.lsos.length;x++)
		{
			objGlobal.lsos[x].file.remove(false);
		}
		objGlobal.DeleteAllFromTree(objGlobal.lsosTree, objGlobal.lsosTreeView, objGlobal.lsos, objGlobal.deletedLsos, "removeLso", "removeAllLsos");
		objGlobal.SetLsoProperties("", "", "", "");
	},

	SetLsoProperties: function(lso_name, lso_parent, lso_size, lso_path)
	{
		document.getElementById("lso_name").value = lso_name;
		document.getElementById("lso_parent").value = lso_parent;
		document.getElementById("lso_size").value = lso_size;
		document.getElementById("lso_path").value = lso_path;
	},

// Code below inserted from http://www.fiveanddime.net/firefox-1.5-source/mozilla/toolkit/components/passwordmgr/resources/content/passwordManager.js.html
// as I could not find these functions in the current chrome - Trevor Hobson

	DeleteAllFromTree: function(tree, view, table, deletedTable, removeButton, removeAllButton)
	{
		// remove all items from table and place in deleted table
		for (var i=0; i<table.length; i++)
		{
			deletedTable[deletedTable.length] = table[i];
		}
		table.length = 0;

		// clear out selections
		view.selection.select(-1); 

		// update the tree view and notify the tree
		view.rowCount = 0;

		var box = tree.treeBoxObject;
		box.rowCountChanged(0, -deletedTable.length);
		box.invalidate();

		// disable buttons
		document.getElementById(removeButton).setAttribute("disabled", "true")
		document.getElementById(removeAllButton).setAttribute("disabled","true");
	},

	DeleteSelectedItemFromTree: function(tree, view, table, deletedTable, removeButton, removeAllButton)
	{
		var box = tree.treeBoxObject;

		// Remove selected items from list (by setting them to null) and place in
		// deleted list.  At the same time, notify the tree of the row count changes.
		var selection = box.view.selection;
		var oldSelectStart = table.length;
		box.beginUpdateBatch();

		var selCount = selection.getRangeCount();
		var min = new Object();
		var max = new Object();

		for (var s = 0; s < selCount; ++s)
		{
			selection.getRangeAt(s, min, max);
			var minVal = min.value;
			var maxVal = max.value;

			oldSelectStart = minVal < oldSelectStart ? minVal : oldSelectStart;

			var rowCount = maxVal - minVal + 1;
			view.rowCount -= rowCount;
			box.rowCountChanged(minVal, -rowCount);

			for (var i = minVal; i <= maxVal; ++i)
			{
				deletedTable[deletedTable.length] = table[i];
				table[i] = null;
			}
		}

		// collapse list by removing all the null entries
		for (var j = 0; j < table.length; ++j)
		{
			if (!table[j])
			{
				var k = j;
				while (k < table.length && !table[k])
				k++;
				table.splice(j, k-j);
			}
		}

		box.endUpdateBatch();

		// update selection and/or buttons
		var removeBtn = document.getElementById(removeButton);
		var removeAllBtn = document.getElementById(removeAllButton);

		if (table.length)
		{
			removeBtn.removeAttribute("disabled");
			removeAllBtn.removeAttribute("disabled");

			selection.select(oldSelectStart < table.length ? oldSelectStart : table.length - 1);
		}
		else
		{
			removeBtn.setAttribute("disabled", "true");
			removeAllBtn.setAttribute("disabled", "true");
		}
	},

	GetTreeSelections: function(tree)
	{
		var selections = [];
		var select = tree.view.selection;
		if (select)
		{
			var count = select.getRangeCount();
			var min = new Object();
			var max = new Object();
			for (var i=0; i<count; i++)
			{
				select.getRangeAt(i, min, max);
				for (var k=min.value; k<=max.value; k++)
				{
					if (k != -1)
					{
						selections[selections.length] = k;
					}
				}
			}
		}
		return selections;
	},

	// Split original into two functions (SortTree and SortTreeArray) so can sort the array before applying to tree mainly for FF3 null issue
	SortTree: function(tree, view, table, column, lastSortColumn, lastSortAscending, updateSelection)
	{
		// remember which item was selected so we can restore it after the sort
		var selections = objGlobal.GetTreeSelections(tree);
		var selectedNumber = selections.length ? table[selections[0]].number : -1;

		// sort the table (array)
		var ascending = objGlobal.SortTreeArray(table, column, lastSortColumn, lastSortAscending);

		// restore the selection
		var selectedRow = -1;
		if (selectedNumber>=0 && updateSelection)
		{
			for (var s=0; s<table.length; s++)
			{
				if (table[s].number == selectedNumber)
				{
					// update selection
					// note: we need to deselect before reselecting in order to trigger ...Selected()
					tree.view.selection.select(-1);
					tree.view.selection.select(s);
					selectedRow = s;
					break;
				}
			}
		}

		// display the results
		tree.treeBoxObject.invalidate();
		if (selectedRow >= 0)
		{
			tree.treeBoxObject.ensureRowIsVisible(selectedRow)
		}

		return ascending;
	},

	SortTreeArray: function(table, column, lastSortColumn, lastSortAscending)
	{
		// determine if sort is to be ascending or descending
		var ascending = (column == lastSortColumn) ? !lastSortAscending : true;

		// do the sort
		var compareFunc;
		if (ascending)
		{
			compareFunc = function compare(first, second)
			{
				return objGlobal.CompareLowerCase(first[column], second[column]);
			}
		}
		else
		{
			compareFunc = function compare(first, second)
			{
				return objGlobal.CompareLowerCase(second[column], first[column]);
			}
		}

		table.sort(compareFunc);
		return ascending;
	},

	// Case insensitive string comparator.
	CompareLowerCase: function(first, second)
	{
		var firstLower  = first.toLowerCase();
		var secondLower = second.toLowerCase();

		if (firstLower < secondLower)
		{
			return -1;
		}

		if (firstLower > secondLower)
		{
			return 1;
		}

		return 0;
	},
// End imported code
};

// automatic detection for Flash Apps directory
function GetFlashDir(ForceSearch)
{
	var objectionPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.objection.");
	var LocalFile = new Components.Constructor("@mozilla.org/file/local;1", "nsILocalFile");
	var flashDir = new LocalFile();
	var varDir = new LocalFile();
	if (((!objectionPrefs.prefHasUserValue("flashAppDir"))||(typeof(objectionPrefs.getCharPref("flashAppDir")) == 'undefined'))||ForceSearch)
	{
		if (window.navigator.platform == "Win32")
		{
			var winDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("AppData", Components.interfaces.nsILocalFile);
			flashDir = winDir;
			flashDir.appendRelativePath('\Macromedia');
			flashDir.appendRelativePath('\Flash Player');
			if ((flashDir.exists()) && (flashDir.isDirectory()))
			{
				varDir = flashDir;
				objectionPrefs.setCharPref("flashAppDir", varDir.path);
			}
			else
			{
				flashDir = winDir;
				flashDir.appendRelativePath('\Macromedia');
				if ((flashDir.exists()) && (flashDir.isDirectory()))
				{
					varDir = flashDir;
					objectionPrefs.setCharPref("flashAppDir", varDir.path);
				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
		}
		else if (window.navigator.platform == "MacPPC")
		{
			var macDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("UsrPrfs", Components.interfaces.nsILocalFile);
			flashDir = macDir;
			flashDir.appendRelativePath('Macromedia');
			flashDir.appendRelativePath('Flash Player');
			if ((flashDir.exists()) && (flashDir.isDirectory()))
			{
				varDir = flashDir;
				objectionPrefs.setCharPref("flashAppDir", varDir.path);
			}
			else
			{
				flashDir = macDir;
				flashDir.appendRelativePath('\Macromedia');
				if ((flashDir.exists()) && (flashDir.isDirectory()))
				{
					varDir = flashDir;
					objectionPrefs.setCharPref("flashAppDir", varDir.path);
				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
		}
		else
		{
			var homeDir = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("Home", Components.interfaces.nsILocalFile);
			flashDir = homeDir;
			flashDir.appendRelativePath('.macromedia');
			flashDir.appendRelativePath('Macromedia');
			flashDir.appendRelativePath('Flash \Player');
			if ((flashDir.exists()) && (flashDir.isDirectory()))
			{
				varDir = flashDir;
				objectionPrefs.setCharPref("flashAppDir", varDir.path);
			}
			else
			{
				flashDir = homeDir;
				flashDir.appendRelativePath('.macromedia');
				flashDir.appendRelativePath('Macromedia');
				if ((flashDir.exists()) && (flashDir.isDirectory()))
				{
					varDir = flashDir;
					objectionPrefs.setCharPref("flashAppDir", varDir.path);
				}
				else
				{
					varDir = "";
					objectionPrefs.setCharPref("flashAppDir", "");
				}
			}
		}
	}
	else
	{
		if (objectionPrefs.getCharPref("flashAppDir") != "")
		{
			varDir.initWithPath(objectionPrefs.getCharPref("flashAppDir"));
		}
		else varDir = "";
	}
	return varDir;
}